/*******************************************************************/
/*                                                                 */
/*      Data Transfer to Serial Port(Communicate with SNDS)        */
/*                                                                 */ 
/*-----------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.                    */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*      Programmed by hbahn, from 1998.1.7                         */
/*                                                                 */
/*******************************************************************/
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>
#include "serial.h"

unsigned int 	FileSize() ;
unsigned 	GenCheckSum(FILE *InFile,int SendFileSize) ;
unsigned 	MakeSendFile(char OutFileName[20]) ;
void 		SendData(char SendFileName[20], unsigned SendFileSize) ;
unsigned 	CalcCRC( unsigned CheckSum, int CharData ) ;
void 		SendDataSize(unsigned int SendFileSize) ;
unsigned int 	SendDataContent(FILE *SendFileStream, unsigned SendFileSize) ;
void 		SendCRCData(unsigned int SendCRCValue) ;
void 		wait(int loop) ;


extern 	void 	SetupComPort(int);
extern 	void 	InitComPort(int) ;
extern 	void 	Put_Char(char ch) ;
extern 	char 	Get_Char(void) ;


void main(int argc,char *argv[])
{
	char 		*SendFileName ;
	unsigned 	int SendFileSize ;
	unsigned 	int SendCRCValue ;
	int 		PortName ;
	char 		c ;
	long unsigned int t;
	time_t 		t_start, t_end;
	FILE 		*SendFileStream ;

	printf("\n +---------------------------------------------+") ;
	printf("\n |       Down Load program for SNDS board      |") ;
	printf("\n |       Usage : sftp 1[/2] <filename>         |") ;
	printf("\n | (Default:COM1,38400,8 Data,1 Stop,NoParity) |") ;
	printf("\n +---------------------------------------------+") ;

	if (argc < 3) {
		printf("\nUsage : sftp 1[/2] <File Name>\n") ;
		exit(0);
        }
	else {
		SendFileName = argv[2] ;
		printf("\n\n $ Inputfile name : %s", SendFileName) ; 
	}

	// Initialize COM Port
	PortName = atoi(argv[1]);
       
        do {
              printf("\n $ Need to Change communication parameters?[Y/N]");
              c = getchar();
              if(c=='Y'||c=='y') SetupComPort(PortName);
              if(c=='N'||c=='n') InitComPort(PortName);
        } while( c!='Y'&& c!='y'&& c!='N'&& c!='n'); 
	
	// Send Data
	if( ( SendFileStream = fopen(SendFileName,"r+b") ) != NULL)
	{		  
		// Send File Size
		SendFileSize = FileSize(SendFileStream) - 2 ;

		printf("\n Transferring '%s'(%d Byte) to SNDS ....",
			SendFileName,SendFileSize) ;

		SendDataSize(SendFileSize) ;

                /* Delay time for SNDS  */
                t_start = time(NULL);
                do   { 			//wait 5 sec.
                     t_end = time(NULL);
                     t=(long unsigned int)difftime(t_end, t_start);
                } while(t<3);
                printf("wait %dsec\n",t);

		// Send Data Size and CRC Calcuration
		SendCRCValue = SendDataContent(SendFileStream,SendFileSize) ;

		// Send CRC Data 
		SendCRCData(SendCRCValue) ;

		fclose(SendFileStream) ;  
		printf("\n\n Data Transfer Finished \n\n");
		
	} else   
		printf("\n'SendFile' not found\n") ;	
}

// Send Data Size :  4 Byte
void SendDataSize(unsigned int SendFileSize) 
{
	char SendFileSize_Data[8] ;
	int i ;
	
	for (i=0 ; i<8 ; i++) 
		SendFileSize_Data[i] = ((SendFileSize>>i*4) & 0xF) ;

	for (i=7 ; i>=0 ; i--)
	{
	   if ((SendFileSize_Data[i] >= 0x0) && (SendFileSize_Data[i] <= 0x9))
		SendFileSize_Data[i] = SendFileSize_Data[i] + '0' ;
	   else if ((SendFileSize_Data[i] >= 0xa) && (SendFileSize_Data[i] <= 0xf))
		SendFileSize_Data[i] = SendFileSize_Data[i] + 0x57 ;

	Put_Char(SendFileSize_Data[i]) ;
	}
}


// Calcurate File Size
unsigned int FileSize(FILE *InFile)
{
   unsigned long InFileStart, InFileEnd, InFileSize ;

   fseek(InFile,0L,SEEK_END) ;
   InFileEnd = ftell(InFile) ;
   fseek(InFile,0L,SEEK_SET) ;
   InFileStart = ftell(InFile) ;
   InFileSize = InFileEnd - InFileStart ;  

   return InFileSize ;
}


// Send Data Contents
unsigned int SendDataContent(FILE *SendFileStream, unsigned SendFileSize) 
{
	unsigned int CheckSum = 0xFFFF ;
	char ch ;
	int i=0 ;

	while(SendFileSize--)	
	{					 
		ch=getc(SendFileStream) ;	
		CheckSum=CalcCRC(CheckSum,ch) ;
		wait(500) ;
		Put_Char(ch) ;

		// Print Data Transfering Status
		if (i==500) 
		{ 
			putc('#',stdout) ;
			i=0 ;
		}
		else i++ ;
	}

	return CheckSum ;
}

// CRC Generation
unsigned CalcCRC( unsigned CheckSum, int CharData )
{ 
	int j;
	
	for( j = 0; j < 8; j++ )  
	{    
		CheckSum <<= 1;           // Module polynomial division
		if( CharData & 0x0080 ) CheckSum ^= CCITT_POLYNOM;   
		CharData <<= 1;  
	}  
	return CheckSum ;
}

// Send CRC Data : 4 Byte
void SendCRCData(unsigned int SendCRCValue) 
{
	int i ;
	char CRC_Value[8] ;

	for (i=0 ; i<8 ; i++) 
		CRC_Value[i] = ((SendCRCValue>>i*4) & 0xF) ;

	for (i=7 ; i>=0 ; i--)
	{
	   if ((CRC_Value[i] >= 0x0) && (CRC_Value[i] <= 0x9))
		CRC_Value[i] = CRC_Value[i] + '0' ;
	   else if ((CRC_Value[i] >= 0xa) && (CRC_Value[i] <= 0xf))
		CRC_Value[i] = CRC_Value[i] + 0x57 ;
	
	   Put_Char(CRC_Value[i]) ;
	}
}

void wait(int loop)
{
	while (loop--) ;
}
