/*******************************************************************/
/*                                                                 */
/*      Data Transfer to Serial Port(Communicate with SNDS)        */
/*                                                                 */ 
/*-----------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.                    */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*      Programmed by hbahn, from 1998.1.7                         */
/*                                                                 */
/*******************************************************************/




#include <conio.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>
#include "serial.h"

#define DEBUG 		0

#define FALSE           0
#define TRUE           (!FALSE)

#define NOERROR         0       /* No error               */
#define BUFOVFL         1       /* Buffer overflowed      */

#define ESC             0x1B    /* ASCII Escape character */
#define ASCII           0x007F  /* Mask ASCII characters  */
#define SBUFSIZ         0x4000  /* Serial buffer size     */


int            portbase        = 0;

void  	SetSerial();
void  	SetPort(int Port);
void  	SetOthers(int Parity, int Bits, int StopBit);
void  	SetSpeed(int Speed);

void 	SetupComPort(int /*port*/);
void 	InitComPort(int) ;
void 	Init_Serial(void);
char 	DisableSioInt(void) ;
void 	EnableSioInt(char OldIerValue) ;
void 	Setup_Serial(int Port, int Speed, int Parity, int Bits, int StopBit);
void 	Put_Char(char ch) ;
char 	Get_Char(void) ;
void 	TransferringData(char SendData) ;
char 	GetteringData(void) ;


// Transfering Data Section
void TransferringData(char SendData)
{
	Put_Char(SendData) ;
}

// Gettering Data From SNDS
char GetteringData(void)
{
	char GetByte ;
	GetByte = Get_Char() ;
	return GetByte ;
}


// Set up the port 
void Setup_Serial(int Port, int Speed, int Parity, int Bits, int StopBit)
{
	SetPort(Port);
	SetSpeed(Speed) ;
	SetOthers(Parity, Bits, StopBit) ;
}

// Set the port number to use
void SetPort(int Port)
{    
	switch (Port)
	{
      	case COM1 : portbase = COM1BASE; break;
      	case COM2 : portbase = COM2BASE; break;
      	default   : portbase = COM1BASE;    	
	}
}

// Set other communications parameters
void SetOthers(int Parity, int Bits, int StopBit)
{
    	int setting;
		
    	setting  = Bits-5;
    	setting |= ((StopBit == 1) ? 0x00 : 0x04);
    	setting |= Parity;
	
    	_outp((short)(portbase+LCR), setting);
}


//Setup Serial Port Speed
void SetSpeed(int Speed)
{
    	char		c;
    	int		divisor;
	
    	divisor = (int)(115200L/Speed);
		
    	c = _inp((short)(portbase+LCR));
		_outp((short)(portbase+LCR), (c | 0x80)); // Set DLAB
    	_outp((short)(portbase+DLL), (divisor & 0x00FF));
    	_outp((short)(portbase+DLH), ((divisor >> 8) & 0x00FF));
    	_outp((short)(portbase+LCR), c);          // Reset DLAB 
		
}

// Character Out to SNDS
void Put_Char(char ch)
{
    long int  timeout = 0x0000FFFFL;

    // Wait for transmitter to clear 
    while ((_inp((short)(portbase+LSR)) & XMTRDY) == 0)
        if (!(--timeout)) printf("\nTransmit timeout error occured") ;
			
    _outp((short)(portbase+TXR), ch);
  
}

// Get Character from SNDS
char Get_Char(void)
{
	char ch ;

	// Waiting Receive Data		  
	while ((_inp((short)(portbase+LSR)) & RCVRDY) == 0) ;
    	ch=_inp((short)(portbase+RXR)) ;
	return ch ;
}


void InitComPort(int port)
{
    	// Communications parameters
	int        speed    = 38400L;
	int        parity   = NO_PARITY;
	int        bits     = 8;
	int        stopbits = 1;

	Setup_Serial(port, speed, parity, bits, stopbits);
}


void SetupComPort(int port)
{
    	// Communications parameters
	int        speed;
	int        parity   = NO_PARITY;
	int        bits     = 8;
	int        stopbits = 1;

	char        ch;

        //printf("\n\n\n");
        printf("\n\n << SetUp Communication parameter >>\n");
        printf("\n     >> Available Baud Rate <<");
        printf("\n   +-----------------------------+");
        printf("\n   | 1. 9600  bps | 2. 19200 bps |");
        printf("\n   | 3. 38400 bps | 4. 57600 bps |");
	printf("\n   | 5. 115200bps | 6. 230400bps |");
	printf("\n   +-----------------------------+");

        do {
              printf("\n\n -> Select Number : ");
              ch = getchar();
        } while(!isdigit(ch));

        switch(ch) {
            case '1': speed = 9600L; break;
            case '2': speed = 19200L; break;
            case '3': speed = 38400L; break;
            case '4': speed = 57600L; break;
            case '5': speed = 115200L; break;
            case '6': speed = 230400L; break;
            default : speed = 38400L; 
       }
       
       printf("\n $$ Selected Baud Rate is %d BPS\n\n",speed);
       getchar(); 

       Setup_Serial(port, speed, parity, bits, stopbits);
}


char DisableSioInt(void)
{
	char OldIerValue ;
	
	// Save Old Interrupt Register Value
	OldIerValue = _inp((short)(portbase+LCR));

	//Interrupt Disable
	return OldIerValue ;
}


void EnableSioInt(char OldIerValue)
{
	//Interrupt Enable
	_outp((short)(portbase+IER), OldIerValue);
	
}
    			
